#include <libmp/all.h>
#include <stdarg.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <time.h>
#include <unistd.h>

#include "intl.h"
#include "log.h"

void package_list(lm_pkg_data_t *pkg, ssize_t *save, ssize_t max) {
  ssize_t cur = 0;

  if (*save == 0) {
    printf("    ");
    *save = 4;
  }

  if (max == 0) {
    struct winsize barwin = {0};
    ioctl(STDOUT_FILENO, TIOCGWINSZ, &barwin);
    max = barwin.ws_col;
  }

  cur += strlen(pkg->name);
  cur += strlen(pkg->version);
  *save += cur += 3;

  if (cur > max)
    goto end;

  if (*save > max) {
    printf("\n");
    *save = 0;
    return package_list(pkg, save, max);
  }

end:
  printf(FG_BOLD "%s" FG_RESET "_" FG_BOLD FG_GREEN "%s" FG_RESET, pkg->name, pkg->version);
  printf("  ");
}

void info(const char *msg, ...) {
  va_list args;
  va_start(args, msg);

  printf(FG_BOLD FG_BLUE ">>> " FG_RESET);
  vprintf(msg, args);
  printf(FG_RESET "\n");

  va_end(args);
}

void error(const char *msg, ...) {
  va_list args;
  va_start(args, msg);

  printf(FG_BOLD FG_RED ">>> " FG_RESET);
  vprintf(msg, args);
  printf(FG_RESET "\n");

  va_end(args);
}

void success(const char *msg, ...) {
  va_list args;
  va_start(args, msg);

  printf(FG_BOLD FG_GREEN ">>> " FG_RESET);
  vprintf(msg, args);
  printf(FG_RESET "\n");

  va_end(args);
}

void input(const char *msg, ...) {
  va_list args;
  va_start(args, msg);

  printf(FG_BOLD FG_CYAN ">>> " FG_RESET);
  vprintf(msg, args);
  printf(FG_RESET);

  va_end(args);
}

bool yesno(const char *msg) {
  char *yes[] = {_("y"), _("Y")};
  char *no[]  = {_("n"), _("N")};

  char question[strlen(msg) + 12], c;
  sprintf(question, _("%s [y/N] "), msg);

  while (true) {
    input(question);

    int c = getchar();
    if (c == '\n')
      return false;
    getchar();

    for (int i = 0; i < sizeof(yes) / sizeof(char *); i++) {
      if (yes[i][0] == c)
        return true;
    }

    for (int i = 0; i < sizeof(no) / sizeof(char *); i++) {
      if (no[i][0] == c)
        return false;
    }

    error(_("Please answer with y/n"));
  }
}

bool bar(float cur, float max) {
  if (cur <= 0 || max <= 0 || cur > max)
    return false;

  float per = 100 * (cur / max);
  if (per > 100) {
    return false;
  }

  struct winsize barwin = {0};
  char           perc[20];

  ioctl(STDOUT_FILENO, TIOCGWINSZ, &barwin);
  sprintf(perc, " %%%.f", per);

  int size = barwin.ws_col - (3 + strlen(perc));
  if (size <= 0) {
    return false;
  }

  int prog = size * (cur / max);
  if (prog > size)
    return true;

  printf("\e[?25l");

  printf("\r" FG_BOLD "[");
  for (int i = 0; i < prog; i++) {
    printf("#");
  }
  printf(FG_BLUE "#" FG_RESET FG_BOLD);

  for (int i = prog; i < size; i++) {
    printf(" ");
  }
  printf("]" FG_RESET);

  printf(FG_BOLD FG_BLUE "%s" FG_RESET "\r", perc);
  return true;
}

void bar_free() {
  struct winsize barwin = {0};
  ioctl(STDOUT_FILENO, TIOCGWINSZ, &barwin);

  printf("\r");
  for (int i = 0; i < barwin.ws_col; i++)
    printf(" ");
  printf("\r");

  printf("\e[?25h");
}
